#ifndef _global_h_
#define _global_h_
#include "compat.h"
#include "errors.h"

/* return codes */

#define RET_ERR			2
#define RET_PARSER_ERR		1	/* soft error, don't exit */
#define RET_OK			0
#define RET_EXIT		-1	/* no error, but exit */
#define	RET_INTERNAL_ERR	-2	/* hard error - exit */
#define RET_MEM_ERR		-3	/* memory allocation error - abort */

/* justification types */

#define J_NONE		0	/* justification */
#define J_LEFT		1
#define J_RIGHT		2
#define J_CENTR		3

/* wrap types */

#define WRAP_NONE	0
#define WRAP_TRUNC	1
#define WRAP_SIZE	2
#define WRAP_WORD	3
#define WRAP_LINE	4

/* numformat codes */

#define NUMF_INT	1
#define NUMF_FLOAT	2

#define NULL_char	(char *)0

#ifdef TRUE
#undef TRUE
#else
#define TRUE	1
#endif

#ifdef FLASE
#undef FALSE
#else
#define FALSE	0
#endif


#define SKIP_SPACES(x)	while ( *x == ' ' || \
			*x == '\t' || \
			*x == '\n' || \
			*x == '\r') x++

#define SKIP_WORD(x)	while (*x != '\0' && \
			*x != ' ' && \
			*x != '\t' && \
			*x != '\n' && \
			*x != '\r') x++

#define __ARGS(X)

/* Global variables *******************************************/

#ifdef _globals_c_
#define EXTL
#define EXTR(X)		X
#else
#define EXTL		extern
#define EXTR(X)

extern char	*WrapType[];
extern char	*JustType[];
#endif

EXTL int	SQLsock		EXTR( = -1);
EXTL int	Connected	EXTR( = FALSE);
EXTL int	Initial		EXTR( = TRUE);
EXTL int	HostChanged	EXTR( = TRUE);
EXTL char	Qbuf[EXTR(QBUF_SIZE + 2)];
EXTL int	PageEnd		EXTR( = DEF_PAGE_HEIGHT - DEF_MARGIN_BOT - 1);
EXTL int	ScreenEnd	EXTR( = DEF_SCREEN_SIZE - 3);

/* Default values */

EXTL char	def_host[EXTR(MAX_HOSTNAME+1)];
EXTL char	def_dbname[EXTR(MAX_NAME_LEN+1)];
EXTL int	def_out_messag	EXTR( = TRUE);
EXTL int	def_out_report	EXTR( = TRUE);
EXTL int	def_in_echo	EXTR( = FALSE);
EXTL char	def_fprompt[EXTR(MAX_PROMPT_LEN+1)];
EXTL char	def_sprompt[EXTR(MAX_PROMPT_LEN+1)];
EXTL char	def_hline_char	EXTR( = DEF_HLINE_CHAR);
EXTL char	def_fline_char	EXTR( = DEF_FLINE_CHAR);
EXTL int	def_endbar	EXTR( = TRUE);
EXTL int	def_heading	EXTR( = TRUE);
EXTL int	def_page_width	EXTR( = DEF_PAGE_WIDTH);
EXTL int	def_page_size	EXTR( = DEF_PAGE_HEIGHT);
EXTL int	def_page_break	EXTR( = -1);
EXTL int	def_margin_top	EXTR( = DEF_MARGIN_TOP);
EXTL int	def_margin_bot	EXTR( = DEF_MARGIN_BOT);
EXTL int	def_margin_left	EXTR( = 0);
EXTL int	def_buffers	EXTR( = DEF_NUM_BUFFERS);
EXTL char	*def_editor;
EXTL int	def_pause	EXTR( = TRUE);
EXTL int	def_scrsize	EXTR( = DEF_SCREEN_SIZE);
EXTL char	*def_suf_cmd	EXTR( = NULL_char);
EXTL char	*def_suf_lst	EXTR( = NULL_char);
EXTL char	*def_cmddir	EXTR( = NULL_char);
EXTL char	*def_spoolfile	EXTR( = NULL_char);
EXTL char	*def_numformat	EXTR( = NULL_char);
EXTL int	def_numwidth	EXTR( = DEF_NUM_WIDTH);
EXTL char	*def_currency	EXTR( = NULL_char);
EXTL char	*def_shell	EXTR( = NULL_char);
EXTL int	def_wrap	EXTR( = WRAP_TRUNC);
EXTL int	def_maxrows	EXTR( = 0);
EXTL char	def_colsep	EXTR( = 0);
EXTL char	def_rowsep	EXTR( = 0);

#undef EXTL
#undef EXTR

/* Functions ****************************************************************/

/* misc.c */
extern char	*makenumformat();
extern int	max();
extern char	*makefilename();
extern char	*prependdir();
extern char	*copy_str();
extern void	free_char();

/* commands.c */
extern int	scanner();

/* query.c */
extern int	doquery();

/* msgs.c */
extern void	Err();

/* io.c */
extern void	Out();
extern void	OutSilent();
extern void	OutReport();
extern void	ReportOut();
extern void	ReportEnd();
extern void	Prompt();
extern char	*Input();
extern int	Inset();
extern int	InFile();
extern int	SpoolOpen();
extern void	SpoolClose();
extern int	outfile();
extern char	*edit();
extern int	save();

/* sql.c */
extern int	connect_db();
extern int	disconnect_db();

#endif /*header*/
