#include <stdio.h>
#include <string.h>
#include "config.h"
#include "compat.h"
#include "global.h"
#include "commands.h"
#include "buffers.h"

static void	set_show();
static int	do_pagewidth();
static int	do_pagesize();
static int	do_pagebreak();
static int	do_numwidth();
static int	do_numformat();
static int	do_fprompt();
static int	do_sprompt();
static int	do_underline();
static int	do_endbar();
static int	do_heading();
static int	do_out_messag();
static int	do_out_report();
static int	do_in_echo();
static int	do_host();
static int	do_editor();
static int	do_pause();
static int	do_scrsize();
static int	do_cmddir();
static int	do_currency();
static int	do_wrap();
static int	do_maxrows();
static int	do_colsep();
static int	do_rowsep();

static int	margin_scmd();
static int	do_topm();
static int	do_botm();
static int	do_leftm();

static int	suffix_scmd();
static int	do_sufcmd();
static int	do_suflst();

static Command SetCmds[] = {
	"buf",		2,	buf_num_cmd,
	"cur",		2,	do_currency,
	"echo",		2,	do_in_echo,    /*sql+ echoset*/
	"edit",		2,	do_editor,
	"endb",		2,	do_endbar,

	"headi",	2,	do_heading,    /*sql+ heading*/
	"host",		2,	do_host,

	"lib",		2,	do_cmddir,
	"lin",		2,	do_pagewidth,  /*sql+ linesize*/

	"mar",		2,	margin_scmd,
	"max",		2,	do_maxrows,
	"mes",		2,	do_out_messag,

	"numf",		2,	do_numformat,  /*sql+ numformat*/
	"numw",		2,	do_numwidth,   /*sql+ numwidth*/

	"pageb",	2,	do_pagebreak,
	"pageh",	2,	do_pagesize,
	"pagel",	2,	do_pagesize,
	"pages",	2,	do_pagesize,   /*sql+ pagesize*/
	"pagew",	2,	do_pagewidth,
	"pau",		2,	do_pause,
	"prom",		2,	do_fprompt,

	"rep",		2,	do_out_report,
	"rowsep",	2,	do_rowsep,

	"scr",		2,	do_scrsize,
	"sep",		2,	do_colsep,
	"sprom",	2,	do_sprompt,
	"suf",		2,	suffix_scmd,

	"und",		2,	do_underline,  /*sql+ underline*/

	"wra",		2,	do_wrap,
	NULL_char,	0,	NULL,
};

static Command SetSufCmds[] = {
	"com",		2,	do_sufcmd,
	"cmd",		2,	do_sufcmd,
	"sql",		2,	do_sufcmd,
	"lst",		2,	do_suflst,
	"lis",		2,	do_suflst,
	NULL_char,	0,	NULL,
};

static Command SetMarginCmds[] = {
	"bot",		2,	do_botm,
	"lef",		2,	do_leftm,
	"top",		2,	do_topm,
	NULL_char,	0,	NULL,
};


static char	string[128];

/****************************************************************************/

int
set_cmd(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	if(argc < 2){
		set_show();
		return(RET_OK);
	}
	return(subcmd(SetCmds,argc,argv,p));
}



static int
do_pagebreak(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	i;

	if(strcasecmp(argv[1],"on") == 0)
		def_page_break = -1;
	else if (strcasecmp(argv[1],"off") == 0)
		def_page_break = 0;
	else {
		i = atoi(argv[1]);
		if(i < 1 || i > 255){
			Err(ErrUpagebreakinv,NULL_char);
			return(RET_ERR);
		}
		def_page_break = i;
	}

	return(RET_OK);
}

static int
do_pagewidth(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	i;

	if((i = atoi(argv[1])) < 5){
		Err(ErrUpage_w_small,NULL_char);
		return(RET_ERR);
	}

	if(i > MAX_PAGE_WIDTH){
		sprintf(string,"Max - %d",MAX_PAGE_WIDTH);
		Err(ErrUpage_w_large,string);
		return(RET_ERR);
	}

	def_page_width = i;

	return(RET_OK);
}

static int
do_pagesize(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	i,min,h = 0;


	if(def_heading)
		h = 2;

	min = def_margin_top + def_margin_bot + def_endbar + h +1;

	if((i = atoi(argv[1])) < min){
		sprintf(string,"%s",min);
		Err(ErrUpage_s_small,string);
		return(RET_ERR);
	}

	def_page_size = i;
	PageEnd = def_page_size - def_margin_bot - def_endbar;

	return(RET_OK);
}

static int
do_numwidth(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	i;

	if((i = atoi(argv[1])) < 1){
		Err(ErrUnumw_small,NULL_char);
		return(RET_ERR);
	}

	if(i > MAX_NUM_WIDTH){
		sprintf(string,"Max - %d",MAX_NUM_WIDTH);
		Err(ErrUnumw_large,string);
		return(RET_ERR);
	}

	def_numwidth = i;

	return(RET_OK);
}

static int
do_numformat(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	char *f;

	/* check syntax */
	if((f = makenumformat(argv[1],NUMF_INT)) == NULL_char)
		return(RET_ERR);
	free_char(f);

	/* check syntax */
	if((f = makenumformat(argv[1],NUMF_FLOAT)) == NULL_char)
		return(RET_ERR);
	free_char(f);

	free_char(def_numformat);

	if((def_numformat = (char *)malloc(strlen(argv[1])+1)) == NULL){
		Err(ErrIalloc,NULL_char);
		return(RET_MEM_ERR);
	}

	strcpy(def_numformat,argv[1]);

	return(RET_OK);
}


static int
do_fprompt(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{

	if(strlen(argv[1]) > MAX_PROMPT_LEN){
		Err(ErrUprompt_large,"Max - 20");
		return(RET_ERR);
	}
	strcpy(def_fprompt,argv[1]);

	return(RET_OK);
}

static int
do_sprompt(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{

	if(strlen(argv[1]) > MAX_PROMPT_LEN){
		Err(ErrUprompt_large,"Max - 20");
		return(RET_ERR);
	}
	strcpy(def_sprompt,argv[1]);

	return(RET_OK);
}

static int
do_underline(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int i;

	if(boolcmd(argv[1],&i) == RET_OK){
		if(i == TRUE)
			def_hline_char = DEF_HLINE_CHAR; /* ON */
		else
			def_hline_char = ' ';		/* OFF */
	} else
		def_hline_char = argv[1][0];		/* new char */

	return(RET_OK);
}

static int
do_endbar(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int h = 0;

	if(boolcmd(argv[1],&def_endbar) == RET_ERR){
		Err(ErrUsynBool,NULL_char);
		return(RET_ERR);
	}

	if(def_heading)
		h = 2;

	if((def_margin_top+h+def_endbar+def_margin_bot) >= def_page_size){
		Err(ErrUendbarno,NULL_char);
		def_endbar = FALSE;
		return(RET_ERR);
	}

	PageEnd = def_page_size - def_margin_bot - def_endbar;
	ScreenEnd = def_scrsize - def_endbar - 2;

	return(RET_OK);
}

static int
do_heading(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	if(boolcmd(argv[1],&def_heading) == RET_ERR){
		Err(ErrUsynBool,NULL_char);
		return(RET_ERR);
	}

	if(!def_heading)
		return(RET_OK);

	if((def_margin_top+2+def_endbar+def_margin_bot) >= def_page_size){
		Err(ErrUheadingno,NULL_char);
		def_heading = FALSE;
		return(RET_ERR);
	}

	return(RET_OK);
}

static int
do_out_messag(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	if(boolcmd(argv[1],&def_out_messag) == RET_ERR){
		Err(ErrUsynBool,NULL_char);
		return(RET_ERR);
	}
	return(RET_OK);
}

static int
do_out_report(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	if(boolcmd(argv[1],&def_out_report) == RET_ERR){
		Err(ErrUsynBool,NULL_char);
		return(RET_ERR);
	}
	return(RET_OK);
}

static int
do_in_echo(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	if(boolcmd(argv[1],&def_in_echo) == RET_ERR){
		Err(ErrUsynBool,NULL_char);
		return(RET_ERR);
	}
	return(RET_OK);
}

static int
do_host(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	if(strlen(argv[1]) > MAX_HOSTNAME){
		sprintf(string,"Max - %d",MAX_HOSTNAME);
		Err(ErrUhostnamelong,string);
		return(RET_ERR);
	}
	HostChanged = TRUE;
	strcpy(def_host,argv[1]);
	return(RET_OK);
}

static int
do_editor(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	size, append = 0;

	size = strlen(argv[1]);

	if(strchr(argv[1],'%') == NULL){
		size += 3; append = 1;
	}

	free_char(def_editor);

	if((def_editor = (char *)malloc(size +1)) == NULL){
		Err(ErrIalloc,"(editor)");
		return(RET_MEM_ERR);
	}

	if(append)
		sprintf(def_editor,"%s %%s",argv[1]);
	else
		strcpy(def_editor,argv[1]);

	return(RET_OK);
}


static int
do_pause(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	if(boolcmd(argv[1],&def_pause) == RET_ERR){
		Err(ErrUsynBool,NULL_char);
		return(RET_ERR);
	}
	return(RET_OK);
}

static int
do_scrsize(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int size;

	if((size = atoi(argv[1])) < 7){
		Err(ErrUscrsize,"Min - 7");
		return(RET_ERR);
	}
	def_scrsize = size;
	ScreenEnd = def_scrsize - def_endbar - 2;
	return(RET_OK);
}

static int
do_cmddir(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	size;

	free_char(def_cmddir);

	if((size = strlen(argv[1])) == 0)
		return(RET_OK);

	if((def_cmddir = (char *)malloc(size +1)) == NULL){
		Err(ErrIalloc,"(cmddir)");
		return(RET_MEM_ERR);
	}

	strcpy(def_cmddir,argv[1]);

	return(RET_OK);
}

static int
do_currency(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	size;
	char	*cp;

	size = strlen(argv[1]);
	if(size > MAX_NUM_WIDTH){
		Err(ErrUstringlong,NULL_char);
		return(RET_ERR);
	}

	free_char(def_currency);

	if((def_currency = (char *)malloc(size +1)) == NULL){
		Err(ErrIalloc,"(currency)");
		return(RET_MEM_ERR);
	}
	strcpy(def_currency,argv[1]);
	return(RET_OK);
}

static int
do_wrap(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	switch(argv[1][0]){
	case 'l':
	case 'L':
		def_wrap = WRAP_LINE;
		break;
	case 's':
	case 'S':
		def_wrap = WRAP_SIZE;
		break;
	case 't':
	case 'T':
		def_wrap = WRAP_TRUNC;
		break;
	case 'w':
	case 'W':
		def_wrap = WRAP_WORD;
		break;
	default:
		Err(ErrUsynwrap,NULL_char);
		return(RET_ERR);
	}
	return(RET_OK);
}

static int
do_maxrows(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	def_maxrows = atoi(argv[1]);
	return(RET_OK);
}

static int
do_colsep(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{

	if (strcasecmp(argv[1],"off") == 0)
		def_colsep = 0;
	else
		def_colsep = argv[1][0];

	return(RET_OK);
}

static int
do_rowsep(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{

	if (strcasecmp(argv[1],"off") == 0)
		def_rowsep = 0;
	else
		def_rowsep = argv[1][0];

	return(RET_OK);
}

/**************************************************************************/

static int
suffix_scmd(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	return(subcmd(SetSufCmds,argc,argv,p));
}

static int
do_sufcmd(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	size;
	char	*cp;

	cp = argv[1];
	if(*cp == '.')
		cp++;

	if(def_suf_cmd != NULL_char){
		free_char(def_suf_cmd);
		def_suf_cmd = NULL_char;
	}

	if((size = strlen(cp)) < 1)
		return(RET_OK);

	if((def_suf_cmd = (char *)malloc(size +1)) == NULL){
		Err(ErrIalloc,"(suf_cmd)");
		def_suf_cmd = NULL_char;
		return(RET_OK);
	}

	strcpy(def_suf_cmd,cp);

	return(RET_OK);
}

static int
do_suflst(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	size;
	char	*cp;

	cp = argv[1];
	if(*cp == '.')
		cp++;

	if(def_suf_lst != NULL_char){
		free_char(def_suf_lst);
		def_suf_lst = NULL_char;
	}

	if((size = strlen(cp)) < 1)
		return(RET_OK);

	if((def_suf_lst = (char *)malloc(size +1)) == NULL){
		Err(ErrIalloc,"(suf_lst)");
		def_suf_lst = NULL_char;
		return(RET_OK);
	}

	strcpy(def_suf_lst,cp);

	return(RET_OK);
}

/**************************************************************************/

static int
margin_scmd(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	return(subcmd(SetMarginCmds,argc,argv,p));
}

static int
do_topm(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	i,max,h = 0;

	if((i  = atoi(argv[1])) < 0){
		Err(ErrUmarginvalid,NULL_char);
		return(RET_ERR);
	}

	if(def_heading)
		h = 2;

	max = def_page_size - def_margin_bot - h - def_endbar - 1;

	if(i > max){
		sprintf(string,"%d",max);
		Err(ErrUmarginbig,string);
		return(RET_ERR);
	}

	def_margin_top = i;
	return(RET_OK);
}

static int
do_botm(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	i,max,h = 0;

	if((i  = atoi(argv[1])) < 0){
		Err(ErrUmarginvalid,NULL_char);
		return(RET_ERR);
	}
	if(def_heading)
		h = 2;

	max = def_page_size - def_margin_top - h - def_endbar - 1;

	if(i > max){
		sprintf(string,"%d",max);
		Err(ErrUmarginbig,string);
		return(RET_ERR);
	}

	def_margin_bot = i;

	PageEnd = def_page_size - def_margin_bot - def_endbar;
	return(RET_OK);
}

static int
do_leftm(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	int	i;
	if((i  = atoi(argv[1])) < 0){
		Err(ErrUmarginvalid,NULL_char);
		return(RET_ERR);
	}
	if(i >= (def_page_width + 3)){
		Err(ErrUmarginvalid,"> PAGEWIDTH + 3");
		return(RET_ERR);
	}
	def_margin_left = i;
	return(RET_OK);
}

/**************************************************************************/

#define NORM(S,V)	sprintf(tmp,S,V);Out(tmp)
#define BOOL(S,V)	sprintf(tmp,S, V ? "ON" : "OFF");Out(tmp)
#define NOCH(S,V,M)	sprintf(tmp,S, V == NULL_char ? M : V);Out(tmp)

static void
set_show()
{
	char *tmp = string;

   Out("\nGeneral.\n");

	NORM("    Host:                      %s\n",def_host);
	sprintf(tmp,
	     "    Database:                  %s%s\n",
			def_dbname[0] == '\0' ? "(not defined)" : def_dbname,
			Connected ? " - Connected" : "");
			Out(tmp);


   Out("\nTerminal output.\n");

	BOOL("    Display messages:          %s\n",def_out_messag);
	BOOL("    Display reports:           %s\n",def_out_report);
	BOOL("    Echo:                      %s\n",def_in_echo);
	BOOL("    Pause:                     %s\n",def_pause);
	NORM("    Screen size:               %d\n",def_scrsize);
	NORM("    Prompt:                    '%s'\n",def_fprompt);
	NORM("    Prompt second:             '%s'\n",def_sprompt);

   Out("\nReport.\n");

	 Out("    Maximum rows to print:     ");
	if(def_maxrows > 0){ 
		sprintf(tmp,"%d\n",def_maxrows);
		Out(tmp);
	} else
		Out("(no limit)\n");

	NORM("    Page widht:                %d\n",def_page_width); 
	NORM("    Page height:               %d\n",def_page_size);
	NORM("    Top margin:                %d\n",def_margin_top);
	NORM("    Bottom margin:             %d\n",def_margin_bot);
	NORM("    Left margin:               %d\n",def_margin_left);
	BOOL("    Print column headings:     %s\n",def_heading);
	BOOL("    Print endbar:              %s\n",def_endbar);
	 Out("    Page break:                ");
	if(def_page_break == 0)
		Out("OFF\n");
	else if(def_page_break == -1)
		Out("ON\n");
	else {
		sprintf(tmp,"ON (character code %d)\n",def_page_break);
		Out(tmp);
	}
	NORM("    Numeric values widht:      %d\n",def_numwidth);
	NORM("    Numeric values format:     '%s'\n",def_numformat); 
	NORM("    Currency:                  '%s'\n",def_currency);
	NORM("    Headings underline:        '%c'\n", def_hline_char);
	 Out("    Columns separator:         ");
	if(def_colsep){
		sprintf(tmp,"'%c'\n",def_colsep);
		Out(tmp);
	}else
		Out("OFF\n");
	 Out("    Rows separator:            ");
	if(def_rowsep){
		sprintf(tmp,"'%c'\n",def_rowsep);
		Out(tmp);
	}else
		Out("OFF\n");
	NORM("    Wrap:                      %s\n",WrapType[def_wrap]);
	NOCH("    Spooling to file:          %s\n",def_spoolfile,"OFF");

   Out("\nMisc.\n");

	NORM("    Editor:                    %s\n",def_editor);
	NORM("    Number of buffers:         %d\n",def_buffers);
	NOCH("    Suffix for command files:  %s\n",def_suf_cmd,"(none)");
	NOCH("    Suffix for listings:       %s\n",def_suf_lst,"(none)");
	NOCH("    Commands library dir.:     %s\n",def_cmddir,"(not set)");

	Out("\n");
}
